# Copyright (c) 2008, Michael Stone
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#  1. Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#  2. Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
#  3. The names of the authors may not be used to endorse or promote
#     products derived from this software without specific prior
#     written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
# GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from os import system, getcwd
from os.path import join, exists
from optparse import OptionParser
from subprocess import check_call
from sys import exit

if exists('/usr/bin/mock'):
    MOCK = ['/usr/bin/mock', '-q', '-r', 'fedora-7-i386', '--no-cleanup-after']
else:
    MOCK = ['/usr/bin/sudo', '/usr/sbin/mock', '-q', '-r', 'fedora-7-i386', '--no-cleanup-after']

class Mock(object):
    def __init__(self, report):
        self.report = report

    def _run_mock(self, args):
        null = open('/dev/null', 'r+')
        try: check_call(MOCK + args, stdout=null, stderr=null)
        except:
            self.report(0, "Mock Error: %s", ' '.join(MOCK+args))
            mock_log_path = '/var/lib/mock/fedora-7-i386/result/root.log'
            puritan_log_path = '/var/lib/mock/fedora-7-i386/root/builddir/results/puritan.log'
            if exists(mock_log_path):
                self.report(0, "%s", open(mock_log_path, 'r').read())
            if exists(puritan_log_path):
                self.report(0, "-----------\nPuritan Log\n-----------\n%s", open(puritan_log_path, 'r').read())
            self.report(0, "To analyze:\nReview traceback.\nsudo chroot /var/lib/mock/fedora-7-i386/root")
            exit(1)

    def chroot(self, cmd, cwd='/'):
        self._run_mock(['--cwd', cwd, '--chroot', ' '.join(cmd)])

    def init(self):
        self._run_mock(['--init'])

    def install(self, pkgs):
        self._run_mock(['--install'] + pkgs)

    def copyin(self, src, tgt):
        self._run_mock(['--copyin', src, join('/builddir', tgt)])

    def copyout(self, src, tgt):
        self._run_mock(['--copyout', join('/builddir', src), tgt])

def main():
    parser = OptionParser(usage="""
%prog [OPTIONS] PURITAN_REPO COMPILATION_COMMIT RESULTS_DIR [CMD]

    CMD ::= build    - run the compilation. (default)
            download - download packages.""")
    parser.add_option('-v',action='count',dest='verbosity',default=0,
                      help='display verbose progress information; repeat for more verbose output.')
    parser.add_option('-q','--quiet',action='store_true',dest='quiet',
                      default=False, help="don't output anything; "+
                      "use exit status to indicate success.")
    opts, args = parser.parse_args()

    verbosity_options = (['-q'] * opts.quiet) + ((['-v'] * opts.verbosity) * (not opts.quiet))
    def report(lvl, msg, *args):
        if opts.quiet: return
        if opts.verbosity >= lvl:
            if len(args):
                print(msg % args)
            else:
                print(msg)

    if len(args) < 3:
        print
        parser.print_help()
        print
        exit(1)

    repo = args[0]
    commit = args[1]
    results_dir = args[2]

    cmd = 'build'
    if len(args) > 3:
        cmd = args[3]

    null = open('/dev/null', 'r+')
    report(1, 'Checking out compiler %s from %s', commit, repo)
    check_call(['/usr/bin/git', 'checkout', '-f', commit], cwd=repo, stdout=null, stderr=null)
    report(1, 'Initing compiler submodules')
    check_call(['/usr/bin/git', 'submodule', 'init'], cwd=repo, stdout=null, stderr=null)
    report(1, 'Updating compiler submodules')
    check_call(['/usr/bin/git', 'submodule', 'update'], cwd=repo, stdout=null, stderr=null)

    pkgs = open(join(repo, 'dependencies'), 'r').read().split()

    m = Mock(report)
    m.init()
    report(1, 'Installing basic packages (%s)', ', '.join(pkgs))
    m.install(pkgs)
    report(1, 'Installing compilation')
    m.copyin(repo, 'compiler')
    report(1, 'Bootstrapping compilation')
    m.chroot(['/builddir/compiler/bootstrap'], cwd='/builddir/compiler')
    report(1, 'Running compilation')
    m.chroot(['/builddir/compiler/main.py'] + verbosity_options + [cmd], cwd='/builddir/compiler')
    report(1, 'Saving results')
    m.copyout('results', results_dir)

if __name__ == '__main__':
    main()
